<?php
/*
	JoomlaXTC JMenu module

	version 1.3.1
	
	Copyright (C) 2009, 2010  Monev Software LLC.	All Rights Reserved.
	
	Based on work from Open Source Matters.

	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
	THIS LICENSE MIGHT NOT APPLY TO OTHER FILES CONTAINED IN THE SAME PACKAGE.
	
	See COPYRIGHT.php for more information.
	See LICENSE.php for more information.
	
	Monev Software LLC
	www.joomlaxtc.com
*/

defined('_JEXEC') or die('Restricted access');

jimport('joomla.base.tree');
jimport('joomla.utilities.simplexml');

/**
 * mod_mainmenu Helper class
 *
 * @static
 * @package		Joomla
 * @subpackage	Menus
 * @since		1.5
 */
class modJxtcMenuHelper
{
	function buildXML($params)
	{
		$menu = new JxtcMenuTree($params);
		$items = &JSite::getMenu();

		// Get Menu Items
		$rows = $items->getItems('menutype', $params->get('menutype'));
		$maxdepth = $params->get('maxdepth',10);

		// Build Menu Tree root down (orphan proof - child might have lower id than parent)
		$user =& JFactory::getUser();
		$ids = array();
		$ids[0] = true;
		$last = null;
		$unresolved = array();
		// pop the first item until the array is empty if there is any item
		if ( is_array($rows)) {
			while (count($rows) && !is_null($row = array_shift($rows)))
			{
				if (array_key_exists($row->parent, $ids)) {
					$row->ionly = $params->get('menu_images_link');
					$menu->addNode($params, $row);

					// record loaded parents
					$ids[$row->id] = true;
				} else {
					// no parent yet so push item to back of list
					// SAM: But if the key isn't in the list and we dont _add_ this is infinite, so check the unresolved queue
					if(!array_key_exists($row->id, $unresolved) || $unresolved[$row->id] < $maxdepth) {
						array_push($rows, $row);
						// so let us do max $maxdepth passes
						// TODO: Put a time check in this loop in case we get too close to the PHP timeout
						if(!isset($unresolved[$row->id])) $unresolved[$row->id] = 1;
						else $unresolved[$row->id]++;
					}
				}
			}
		}
		$buttonClasses=explode(',',$params->get('ButtonClasses'));
		return $menu->toXML($buttonClasses);
	}

	function &getXML($type, &$params, $decorator)
	{
		static $xmls;

		if (!isset($xmls[$type])) {
			$cache =& JFactory::getCache('mod_jxtc_mainmenu');
			$string = $cache->call(array('modJxtcMenuHelper', 'buildXML'), $params);
			$xmls[$type] = $string;
		}

		// Get document
		$xml = JFactory::getXMLParser('Simple');
		$xml->loadString($xmls[$type]);
		$doc = &$xml->document;

		$menu	= &JSite::getMenu();
		$active	= $menu->getActive();
		$start	= $params->get('startLevel');
		$end	= $params->get('endLevel');
		$sChild	= $params->get('showAllChildren');
		$path	= array();

		// Get subtree
		if ($start)
		{
			$found = false;
			$root = true;
			if(!isset($active)){
				$doc = false;
			}
			else{
				$path = $active->tree;
				for ($i=0,$n=count($path);$i<$n;$i++)
				{
					foreach ($doc->children() as $child)
					{
						if ($child->attributes('id') == $path[$i]) {
							$doc = &$child->ul[0];
							$root = false;
							break;
						}
					}

					if ($i == $start-1) {
						$found = true;
						break;
					}
				}
				if ((!is_a($doc, 'JSimpleXMLElement')) || (!$found) || ($root)) {
					$doc = false;
				}
			}
		}

		if ($doc && is_callable($decorator)) {
			$doc->map($decorator, array('end'=>$end, 'children'=>$sChild));
		}
		return $doc;
	}

	function render(&$params, $callback) {
		$live_site = JURI::root();
		$doc =&JFactory::getDocument();
		JHTML::_('behavior.mootools');

		switch ( $params->get( 'menu_style', 'list' ) ) {
			case 'list_flat' :
				$doc->addStyleSheet($live_site."modules/mod_jxtc_jmenu/css/standardmenu.css");
				// Include the legacy library file
				require_once(dirname(__FILE__).DS.'legacy.php');
				jxtcShowHFMenu($params, 1);
				break;

			case 'horiz_flat' :
				$doc->addStyleSheet($live_site."modules/mod_jxtc_jmenu/css/standardmenu.css");
				// Include the legacy library file
				require_once(dirname(__FILE__).DS.'legacy.php');
				jxtcShowHFMenu($params, 0);
				break;

			case 'vert_indent' :
				$doc->addStyleSheet($live_site."modules/mod_jxtc_jmenu/css/standardmenu.css");
				// Include the legacy library file
				require_once(dirname(__FILE__).DS.'legacy.php');
				jxtcShowVIMenu($params);
				break;

			case 'vert_2col' :
				$doc->addStyleSheet($live_site."modules/mod_jxtc_jmenu/css/standardmenu.css");
				// Include the legacy library file
				require_once(dirname(__FILE__).DS.'legacy.php');
				jxtcShowHFMenu($params, 2);
				break;

			default :

			$menuSpeed = $params->get('menuSpeed',300);
			$mooStyle = $params->get('mooStyle','Bounce.easeOut');
			$doc->addScript($live_site."media/system/js/mootools.js");
			switch ( $params->get( 'menu_style', 'list' ) ) {
			case 'horz_moo' :
				// Include the legacy library file
				$doc->addStyleSheet($live_site."modules/mod_jxtc_jmenu/css/moodropin.css");
				$doc->addScriptDeclaration('var menuSpeed = '.$menuSpeed.';var mooStyle = "'.$mooStyle.'";');
				$doc->addScript($live_site."modules/mod_jxtc_jmenu/js/moodropin.js");
				break;
			case 'horz_dropline' :
				// Include the legacy library file
				$doc->addStyleSheet($live_site."modules/mod_jxtc_jmenu/css/moodropline.css");
				$doc->addScriptDeclaration('var menuSpeed = '.$menuSpeed.';var mooStyle = "'.$mooStyle.'";');
				$doc->addScript($live_site."modules/mod_jxtc_jmenu/js/moodropline.js");
				break;
			case 'horz_column' :
				// Include the legacy library file
				$doc->addStyleSheet($live_site."modules/mod_jxtc_jmenu/css/moocolumns.css");
				$doc->addScriptDeclaration('var menuSpeed = '.$menuSpeed.';var mooStyle = "'.$mooStyle.'";');
				$doc->addScript($live_site."modules/mod_jxtc_jmenu/js/moocolumns.js");
				break;
			}
	function _getItemData(&$params, $item) {
		$data = null;

		// Menu Link is a special type that is a link to another item
		$item->name = htmlentities($item->name);
		if ($item->type == 'menulink')
		{
			$menu = &JSite::getMenu();
			if ($newItem = $menu->getItem($item->query['Itemid'])) {
    		$tmp = clone($newItem);
				$tmp->name	 = '<span><![CDATA['.$item->name.']]></span>';
				$tmp->mid	 = $item->id;
				$tmp->parent = $item->parent;
			} else {
				return false;
			}
		} else {
			$tmp = clone($item);
			$tmp->name = '<span><![CDATA['.$item->name.']]></span>';
		}

		$iParams = new JParameter($tmp->params);
		if ($params->get('menu_images') && $iParams->get('menu_image') && $iParams->get('menu_image') != -1) {
			$imgalign = $params->get('menu_images_align', 0) == 1 ? 'right' : 'left';
			$image = '<img src="'.JURI::base(true).'/images/stories/'.$iParams->get('menu_image').'" align="'.$imgalign.'" alt="'.$item->alias.'"  style="display:none"/>';
			if($tmp->ionly){
				 $tmp->name = null;
			 }
		} else {
			$image = null;
		}
		switch ($tmp->type)
		{
			case 'separator' :

				return '<span class="separator">'.$image.$tmp->name.'</span>';
				break;

			case 'url' :
				if ((strpos($tmp->link, 'index.php?') === 0) && (strpos($tmp->link, 'Itemid=') === false)) {
					$tmp->url = $tmp->link.'&amp;Itemid='.$tmp->id;
				} else {
					$tmp->url = $tmp->link;
				}
				break;

			default :
				$router = JSite::getRouter();
				$tmp->url = $router->getMode() == JROUTER_MODE_SEF ? 'index.php?Itemid='.$tmp->id : $tmp->link.'&Itemid='.$tmp->id;
				break;
		}

		// Print a link if it exists
		if ($tmp->url != null)
		{
			// Handle SSL links
			$iSecure = $iParams->def('secure', 0);
			if ($tmp->home == 1) {
				$tmp->url = JURI::base();
			} elseif (strcasecmp(substr($tmp->url, 0, 4), 'http') && (strpos($tmp->link, 'index.php?') !== false)) {
				$tmp->url = JRoute::_($tmp->url, true, $iSecure);
			} else {
				$tmp->url = str_replace('&', '&amp;', $tmp->url);
			}

			switch ($tmp->browserNav)
			{
				default:
				case 0:
					// _top
					$data = $image.'<a href="'.$tmp->url.'">'.$tmp->name.'</a>';
					break;
				case 1:
					// _blank
					$data = $image.'<a href="'.$tmp->url.'" target="_blank">'.$tmp->name.'</a>';
					break;
				case 2:
					// window.open
					$attribs = 'toolbar=no,location=no,status=no,menubar=no,scrollbars=yes,resizable=yes,'.$this->_params->get('window_open');

					// hrm...this is a bit dickey
					$link = str_replace('index.php', 'index2.php', $tmp->url);
					$data = $image.'<a href="'.$link.'" onclick="window.open(this.href,\'targetWindow\',\''.$attribs.'\');return false;">'.$tmp->name.'</a>';
					break;
			}
		} else {
			$data = $image.'<a>'.$tmp->name.'</a>';
		}

		return $data;
	}
//$menuname = $params->get('menutype');
//$menu = &JSite::getMenu();
//$items = $menu->getItems('menutype', $menuname);
//$buttonClasses=explode(',',$params->get('Backcolor','#c0c0c0'));
//$colidx=0;
//$tree=array();
//$info=array();
//foreach ($items as $item) {
//	$id = $item->id;
//	$name = $item->name;
//	$link = $item->link;
//	$parent = $item->parent;
//	$iParams = new JParameter($item->params);
//	$img = $iParams->get('menu_image');
//	$tree[$parent][]=$id;
//	$info[$id]->name = $name;
//	$info[$id]->link = $link;
//	if ($parent == 0) {
//		$info[$id]->color = $buttonClasses[$colidx];
//		if (++$colidx > count($buttonClasses)) $colidx = 0;
//	}
//	if (empty($img)) {
//		$info[$id]->image = '';
//	}
//	else {
//		$info[$id]->image = '<img src="'.JURI::base(true).'/images/stories/'.$img.'" alt="'.$item->alias.'" />';
//	}
//}
//
//function maketree($key=0,$tree,&$info,&$img) {
//	$html = "<ul>\n";
//	foreach ($tree[$key] as $id) {
//		$name = $info[$id]->name;
//		$link = $info[$id]->link;
//		$image = $info[$id]->image;
//		$html .= '<li style="background-color:'.$info[$id]->color.'">';
//		if (array_key_exists($id,$tree)) {
//			$html .= $image."<a href=\"$link\">$name</a>";
//			$html .= maketree($id,$tree,$info,$img);
//		}
//		else {
//			$html .= $image."<a href=\"$link\">$name</a>";
//		}
//		$html .= "</li>\n";
//	}
//	$html .= "</ul>\n";
//	return $html;
//}
//echo maketree(0,$tree,$info,$img);

				// Include the new menu class
				$xml = modJxtcMenuHelper::getXML($params->get('menutype'), $params, $callback);
				if ($xml) {
					$class = $params->get('class_sfx');
					$xml->addAttribute('class', 'menu'.$class);
					if ($tagId = $params->get('tag_id')) {
						$xml->addAttribute('id', $tagId);
					}
					$result = JFilterOutput::ampReplace($xml->toString((bool)$params->get('show_whitespace')));
					$result = str_replace(array('<ul/>', '<ul />'), '', $result);

					$result = str_replace( '{xm}', '<span class="xmenu">', $result );
					$result = str_replace( '{/xm}', '</span>', $result );
					while (($ini = strpos($result,"{xm ")) !== false) {
			
						$fin = strpos($result,"}",$ini);
						$parms=substr($result,$ini+4,$fin-$ini-4);
						list($position,$width,$height)=explode(',',$parms);
						if (!empty($position)) {
							settype($width,'integer');
							settype($height,'integer');
							$width = $width==0 ? '' : 'width:'.$width.'px;';
							$height = $height==0 ? '' : 'height:'.$height.'px;';
							$modules =&JModuleHelper::getModules($position);
							$attribs = array('style'=>'xhtml');
							$positionhtml = "<div class=\"xmenu_position\" style=\"$width $height\">";
							foreach ($modules as $module) {
								$positionhtml .= JModuleHelper::renderModule( $module, $attribs );
							}
						}
						else {
							$positionhtml='';
						}
						$result = substr_replace($result,$positionhtml,$ini,$fin-$ini+1);
					}
					// Fix document title
					
					$doc =&JFactory::getDocument();
					$doctitle = $doc->getTitle();
					while (($ini = strpos($doctitle,"{xm ")) !== false) { // Do module positions
						$fin = strpos($doctitle,"}",$ini);
						$doctitle = substr_replace($doctitle,'',$ini,$fin-$ini+1);
					}
					$doctitle = str_replace( '{xm}', '', $doctitle );
					$doctitle = str_replace( '{/xm}', '', $doctitle );
					$doc->setTitle($doctitle);
			
					echo $result;
				}
				break;
		}
	}
}

/**
 * Main Menu Tree Class.
 *
 * @package		Joomla
 * @subpackage	Menus
 * @since		1.5
 */
class JxtcMenuTree extends JTree
{
	/**
	 * Node/Id Hash for quickly handling node additions to the tree.
	 */
	var $_nodeHash = array();

	/**
	 * Menu parameters
	 */
	var $_params = null;

	/**
	 * Menu parameters
	 */
	var $_buffer = null;

	function __construct(&$params)
	{
		$this->_params		=& $params;
		$this->_root		= new JxtcMenuNode(0, 'ROOT');
		$this->_nodeHash[0]	=& $this->_root;
		$this->_current		=& $this->_root;
	}

	function addNode(&$params, $item)
	{
		// Get menu item data
		$data = $this->_getItemData($params, $item);

		// Create the node and add it
		$node = new JxtcMenuNode($item->id, $item->name, $item->access, $data);

		if (isset($item->mid)) {
			$nid = $item->mid;
		} else {
			$nid = $item->id;
		}
		$this->_nodeHash[$nid] =& $node;
		$this->_current =& $this->_nodeHash[$item->parent];

		if ($item->type == 'menulink' && !empty($item->query['Itemid'])) {
			$node->mid = $item->query['Itemid'];
		}

		if ($this->_current) {
			$this->addChild($node, true);
		} else {
			// sanity check
			JError::raiseError( 500, 'Orphan Error. Could not find parent for Item '.$item->id );
		}
	}

	function toXML(&$buttonClasses)
	{
		// Initialize variables
		$this->_current =& $this->_root;

		// Recurse through children if they exist
		$colidx=0;
		while ($this->_current->hasChildren())
		{
			$this->_buffer .= '<ul>';
			foreach ($this->_current->getChildren() as $child)
			{
				$buttonClasses = is_array($buttonClasses) ? ' class="'.trim($buttonClasses[$colidx++]).'"' : '';
				if ($colidx > count($buttonClasses)) $colidx = 0;
				$this->_current = & $child;
				$this->_getLevelXML(0,$buttonClass);
			}
			$this->_buffer .= '</ul>';
		}
		if($this->_buffer == '') { $this->_buffer = '<ul />'; }
		return $this->_buffer;
	}

	function _getLevelXML($depth,&$buttonClass)
	{
		$depth++;
		// Start the item
		$rel = (!empty($this->_current->mid)) ? ' rel="'.$this->_current->mid.'"' : '';
		$this->_buffer .= '<li access="'.$this->_current->access.'" level="'.$depth.'" id="'.$this->_current->id.'"'.$rel.$buttonClass.'>';

		// Append item data
		$this->_buffer .= $this->_current->link;

		// Recurse through item's children if they exist
		while ($this->_current->hasChildren())
		{
			$this->_buffer .= '<ul>';
			foreach ($this->_current->getChildren() as $child)
			{
				$this->_current = & $child;
				$this->_getLevelXML($depth,$buttonClass);
			}
			$this->_buffer .= '</ul>';
		}

		// Finish the item
		$this->_buffer .= '</li>';
	}

	function _getItemData(&$params, $item) {
		$data = null;

		// Menu Link is a special type that is a link to another item
		if ($item->type == 'menulink')
		{
			$menu = &JSite::getMenu();
			if ($newItem = $menu->getItem($item->query['Itemid'])) {
    			$tmp = clone($newItem);
				$tmp->name	 = '<span><![CDATA['.$item->name.']]></span>';
				$tmp->mid	 = $item->id;
				$tmp->parent = $item->parent;
			} else {
				return false;
			}
		} else {
			$tmp = clone($item);
			$tmp->name = '<span><![CDATA['.$item->name.']]></span>';
		}

		$iParams = new JParameter($tmp->params);
		if ($params->get('menu_images') && $iParams->get('menu_image') && $iParams->get('menu_image') != -1) {
			$imgalign = $params->get('menu_images_align', 0) == 1 ? 'right' : 'left';
			$image = '<img src="'.JURI::base(true).'/images/stories/'.$iParams->get('menu_image').'" align="'.$imgalign.'" alt="'.$item->alias.'"  style="display:none"/>';
			if($tmp->ionly){
				 $tmp->name = null;
			 }
		} else {
			$image = null;
		}
		switch ($tmp->type)
		{
			case 'separator' :
				return '<span class="separator">'.$image.$tmp->name.'</span>';
				break;

			case 'url' :
				if ((strpos($tmp->link, 'index.php?') === 0) && (strpos($tmp->link, 'Itemid=') === false)) {
					$tmp->url = $tmp->link.'&amp;Itemid='.$tmp->id;
				} else {
					$tmp->url = $tmp->link;
				}
				break;

			default :
				$router = JSite::getRouter();
				$tmp->url = $router->getMode() == JROUTER_MODE_SEF ? 'index.php?Itemid='.$tmp->id : $tmp->link.'&Itemid='.$tmp->id;
				break;
		}

		// Print a link if it exists
		if ($tmp->url != null)
		{
			// Handle SSL links
			$iSecure = $iParams->def('secure', 0);
			if ($tmp->home == 1) {
				$tmp->url = JURI::base();
			} elseif (strcasecmp(substr($tmp->url, 0, 4), 'http') && (strpos($tmp->link, 'index.php?') !== false)) {
				$tmp->url = JRoute::_($tmp->url, true, $iSecure);
			} else {
				$tmp->url = str_replace('&', '&amp;', $tmp->url);
			}

			switch ($tmp->browserNav)
			{
				default:
				case 0:
					// _top
					$data = $image.'<a href="'.$tmp->url.'">'.$tmp->name.'</a>';
					break;
				case 1:
					// _blank
					$data = $image.'<a href="'.$tmp->url.'" target="_blank">'.$tmp->name.'</a>';
					break;
				case 2:
					// window.open
					$attribs = 'toolbar=no,location=no,status=no,menubar=no,scrollbars=yes,resizable=yes,'.$this->_params->get('window_open');

					// hrm...this is a bit dickey
					$link = str_replace('index.php', 'index2.php', $tmp->url);
					$data = $image.'<a href="'.$link.'" onclick="window.open(this.href,\'targetWindow\',\''.$attribs.'\');return false;">'.$tmp->name.'</a>';
					break;
			}
		} else {
			$data = $image.'<a>'.$tmp->name.'</a>';
		}

		return $data;
	}
}

/**
 * Main Menu Tree Node Class.
 *
 * @package		Joomla
 * @subpackage	Menus
 * @since		1.5
 */
class JxtcMenuNode extends JNode
{
	/**
	 * Node Title
	 */
	var $title = null;

	/**
	 * Node Link
	 */
	var $link = null;

	/**
	 * CSS Class for node
	 */
	var $class = null;

	function __construct($id, $title, $access = null, $link = null, $class = null)
	{
		$this->id		= $id;
		$this->title	= $title;
		$this->access	= $access;
		$this->link		= $link;
		$this->class	= $class;
	}
}
